@@ -1,8 +1,9 @@
 import collections
 import copy
+import operator
 import os
-import sys
 
+import sphinx.environment
 import sphinx.util
 from sphinx.util.console import bold
 import sphinx.util.docstrings
@@ -217,20 +218,30 @@ class PythonSphinxMapper(SphinxMapperBase):
             PythonException,
         )
     }
-    if sphinx.version_info >= (2, 1):
-        _OBJ_MAP["property"] = PythonMethod
-    else:
-        _OBJ_MAP["property"] = PythonAttribute
+    _OBJ_MAP["property"] = PythonMethod
 
     def __init__(self, app, template_dir=None, url_root=None):
         super(PythonSphinxMapper, self).__init__(app, template_dir, url_root)
 
-        if sys.version_info < (3, 3):
-            self._use_implicit_namespace = False
-        else:
-            self._use_implicit_namespace = (
-                self.app.config.autoapi_python_use_implicit_namespaces
-            )
+        self._use_implicit_namespace = (
+            self.app.config.autoapi_python_use_implicit_namespaces
+        )
+
+    def _need_to_load(self, files):
+        last_files = getattr(self.app.env, "autoapi_source_files", [])
+        self.app.env.autoapi_source_files = files
+
+        last_mtime = getattr(self.app.env, "autoapi_max_mtime", 0)
+        this_mtime = max(os.path.getmtime(file) for _, file in files)
+        self.app.env.autoapi_max_mtime = this_mtime
+
+        if not self.app.config.autoapi_keep_files:
+            return True
+
+        if self.app.env.config_status != sphinx.environment.CONFIG_OK:
+            return True
+
+        return last_files != files or not last_mtime or last_mtime < this_mtime
 
     def _find_files(self, patterns, dirs, ignore):
         for dir_ in dirs:
@@ -251,6 +262,12 @@ class PythonSphinxMapper(SphinxMapperBase):
         shortened, relative path the package/module
         """
         dir_root_files = list(self._find_files(patterns, dirs, ignore))
+        if not self._need_to_load(dir_root_files):
+            LOGGER.debug(
+                "[AutoAPI] Skipping read stage because source files have not changed."
+            )
+            return False
+
         for dir_root, path in sphinx.util.status_iterator(
             dir_root_files,
             bold("[AutoAPI] Reading files... "),
@@ -262,6 +279,8 @@ class PythonSphinxMapper(SphinxMapperBase):
                 data["relative_path"] = os.path.relpath(path, dir_root)
                 self.paths[path] = data
 
+        return True
+
     def read_file(self, path, **kwargs):
         """Read file input into memory, returning deserialized objects
 
@@ -275,7 +294,8 @@ class PythonSphinxMapper(SphinxMapperBase):
                 parsed_data = Parser().parse_file(path)
             return parsed_data
         except (IOError, TypeError, ImportError):
-            LOGGER.warning("Error reading file: {0}".format(path))
+            LOGGER.warning("Unable to read file: {0}".format(path))
+            LOGGER.debug("Reason:", exc_info=True)
         return None
 
     def _resolve_placeholders(self):
@@ -331,12 +351,25 @@ class PythonSphinxMapper(SphinxMapperBase):
             if lines and "autodoc-process-docstring" in self.app.events.events:
                 self.app.emit(
                     "autodoc-process-docstring", cls.type, obj.name, None, None, lines
-                )  # object  # options
+                )
             obj.docstring = "\n".join(lines)
+            self._record_typehints(obj)
 
             for child_data in data.get("children", []):
                 for child_obj in self.create_class(
                     child_data, options=options, **kwargs
                 ):
                     obj.children.append(child_obj)
+
+            # Parser gives children in source order already
+            if self.app.config.autoapi_member_order == "alphabetical":
+                obj.children.sort(key=operator.attrgetter("name"))
+            elif self.app.config.autoapi_member_order == "groupwise":
+                obj.children.sort(key=lambda x: (x.member_order, x.name))
+
             yield obj
+
+    def _record_typehints(self, obj):
+        if isinstance(obj, (PythonClass, PythonFunction, PythonMethod)):
+            annotations = self.app.env.temp_data.setdefault("annotations", {})
+            annotations[obj.id] = obj.obj["annotations"]
